//+------------------------------------------------------------------+
//|                                                       Zigzag.mq4 |
//|                 Copyright  2005-2007, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
//| ZigZag_TRO_MODIFIED_VERSION                                      |
//| MODIFIED BY AVERY T. HORTON, JR. AKA THERUMPLEDONE@GMAIL.COM     |
//| I am NOT the ORIGINAL author 
//  and I am not claiming authorship of this indicator. 
//  All I did was modify it. I hope you find my modifications useful.|
//|                                                                  |
//+------------------------------------------------------------------+

/*
    ZigZag Indicator tracks and connects extreme points of the chart, the distance between these points being equal 
    or higher than the percentage specified for the price scale.

    Depth is the minimal amount of bars where there will not be the second maximum (minimum) 
    
    Deviation pips higher or lower than the previous one. I.e., ZigZag can always diverge, 
    but it can converge (or completely draw together) more than at the Deviation value only after Depth bars. 
    
    Backstep is the minimal amount of bars between maximums/minimums.

    After the ZigZag has captured the lowest point, it starts to search for the turn point until the rollback down 
    from the maximum value exceeds the parameter. As soon as rollover down exceeds the parameter, 
    the second (in this case, the upper) point is considered for captured, and 
    ZigZag starts to search for the third one (in this case, for the lower one), etc.



*/
#property copyright "Copyright  2007, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net/"

#property indicator_chart_window


//---- indicator parameters
extern int ExtDepth=3;
extern int ExtDeviation=5;
extern int ExtBackstep=3;

extern bool Arrows.On = true ;

extern int myChartX   = 300 ;
extern int myChartY   = 300 ;
extern int myCorner   = 0 ;
extern string myFont  = "Courier New" ;
extern int myFontSize = 12 ;
extern color colorHead   = LightGray ;
extern color colorZigZag  = Orange ;

 
//---- PipsToOpen
string Indicator_Name = "MPMMZIGZAG:" ;
int Objs = 0;


//---- indicator buffers
double ZigzagBuffer[];
double HighMapBuffer[];
double LowMapBuffer[];
int level=3; // recounting's depth 
bool downloadhistory=false;

//+--------- TRO MODIFICATION ---------------------------------------+ 

int ZigUp, ZagDn, xWait, ZipTop, ZipBot, ZigDiff, ZigBars, PrevZigBars ;

string symbol, tChartPeriod, ShortName ;  

int     digits, period, win, digits2, n, j, i, k; 
 

double point, point2, value ;
string lbl[15], lbl2[15];



//+------------------------------------------------------------------+
int init()
  {
  

   symbol       =  Symbol() ;
   period       =  Period() ;  
   tChartPeriod =  TimeFrameToString(period) ;
   symbol       =  Symbol() ;
   digits       =  Digits ;
   point        =  Point ;
   ShortName    =  tChartPeriod + " ZigZag("+ExtDepth+","+ExtDeviation+","+ExtBackstep+")" ;
   win          =  0 ;

   if(digits == 5 || digits == 3) { digits = digits - 1 ; point = point * 10 ; } else { digits2 = 0 ; }
  
  
   IndicatorBuffers(3);
//---- drawing settings
 
   

//---- indicator buffers mapping
   SetIndexBuffer(0,ZigzagBuffer);
   SetIndexStyle(0,DRAW_SECTION,STYLE_SOLID,1, colorZigZag);      
   SetIndexBuffer(1,HighMapBuffer);
   SetIndexBuffer(2,LowMapBuffer);
   SetIndexEmptyValue(0,0.0);

//---- indicator short name
   IndicatorShortName(ShortName);
//---- initialization done
   return(0);
  }
//+------------------------------------------------------------------+
int deinit()
  {
 
   for(int i = 0; i < Objs; i++)
     {
      if(!ObjectDelete(Indicator_Name + i))
          Print("error: code #", GetLastError());
     }
     
ObjectsDeleteAll();     
 
   return(0);
  }
 
//+------------------------------------------------------------------+  

int start()
  {
   int i, counted_bars = IndicatorCounted();
   int limit,counterZ,whatlookfor;
   int shift,back,lasthighpos,lastlowpos;
   double val,res;
   double curlow,curhigh,lasthigh,lastlow;

   if (counted_bars==0 && downloadhistory) // history was downloaded
     {
      ArrayInitialize(ZigzagBuffer,0.0);
      ArrayInitialize(HighMapBuffer,0.0);
      ArrayInitialize(LowMapBuffer,0.0);
     }
   if (counted_bars==0) 
     {
      limit=Bars-ExtDepth;
      downloadhistory=true;
     }
   if (counted_bars>0) 
     {
      while (counterZ<level && i<100)
        {
         res=ZigzagBuffer[i];
         if (res!=0) counterZ++;
         i++;
        }
      i--;
      limit=i;
      if (LowMapBuffer[i]!=0) 
        {
         curlow=LowMapBuffer[i];
         whatlookfor=1;
        }
      else
        {
         curhigh=HighMapBuffer[i];
         whatlookfor=-1;
        }
      for (i=limit-1;i>=0;i--)  
        {
         ZigzagBuffer[i]=0.0;  
         LowMapBuffer[i]=0.0;
         HighMapBuffer[i]=0.0;
        }
     }
      
   for(shift=limit; shift>=0; shift--)
     {
      val=Low[iLowest(NULL,0,MODE_LOW,ExtDepth,shift)];
      if(val==lastlow) val=0.0;
      else 
        { 
         lastlow=val; 
         if((Low[shift]-val)>(ExtDeviation*Point)) val=0.0;
         else
           {
            for(back=1; back<=ExtBackstep; back++)
              {
               res=LowMapBuffer[shift+back];
               if((res!=0)&&(res>val)) LowMapBuffer[shift+back]=0.0; 
              }
           }
        } 
      if (Low[shift]==val) LowMapBuffer[shift]=val; else LowMapBuffer[shift]=0.0;
      //--- high
      val=High[iHighest(NULL,0,MODE_HIGH,ExtDepth,shift)];
      if(val==lasthigh) val=0.0;
      else 
        {
         lasthigh=val;
         if((val-High[shift])>(ExtDeviation*Point)) val=0.0;
         else
           {
            for(back=1; back<=ExtBackstep; back++)
              {
               res=HighMapBuffer[shift+back];
               if((res!=0)&&(res<val)) HighMapBuffer[shift+back]=0.0; 
              } 
           }
        }
      if (High[shift]==val) HighMapBuffer[shift]=val; else HighMapBuffer[shift]=0.0;
     }

   // final cutting 
   if (whatlookfor==0)
     {
      lastlow=0;
      lasthigh=0;  
     }
   else
     {
      lastlow=curlow;
      lasthigh=curhigh;
     }
   for (shift=limit;shift>=0;shift--)
     {
      res=0.0;
      switch(whatlookfor)
        {
         case 0: // look for peak or lawn 
            if (lastlow==0 && lasthigh==0)
              {
               if (HighMapBuffer[shift]!=0)
                 {
                  lasthigh=High[shift];
                  lasthighpos=shift;
                  whatlookfor=-1;
                  ZigzagBuffer[shift]=lasthigh;
                  res=1;
                 }
               if (LowMapBuffer[shift]!=0)
                 {
                  lastlow=Low[shift];
                  lastlowpos=shift;
                  whatlookfor=1;
                  ZigzagBuffer[shift]=lastlow;
                  res=1;
                 }
              }
             break;  
         case 1: // look for peak
            if (LowMapBuffer[shift]!=0.0 && LowMapBuffer[shift]<lastlow && HighMapBuffer[shift]==0.0)
              {
               ZigzagBuffer[lastlowpos]=0.0;
               lastlowpos=shift;
               lastlow=LowMapBuffer[shift];
               ZigzagBuffer[shift]=lastlow;
               res=1;
              }
            if (HighMapBuffer[shift]!=0.0 && LowMapBuffer[shift]==0.0)
              {
               lasthigh=HighMapBuffer[shift];
               lasthighpos=shift;
               ZigzagBuffer[shift]=lasthigh;
               whatlookfor=-1;
               res=1;
              }   
            break;               
         case -1: // look for lawn
            if (HighMapBuffer[shift]!=0.0 && HighMapBuffer[shift]>lasthigh && LowMapBuffer[shift]==0.0)
              {
               ZigzagBuffer[lasthighpos]=0.0;
               lasthighpos=shift;
               lasthigh=HighMapBuffer[shift];
               ZigzagBuffer[shift]=lasthigh;
              }
            if (LowMapBuffer[shift]!=0.0 && HighMapBuffer[shift]==0.0)
              {
               lastlow=LowMapBuffer[shift];
               lastlowpos=shift;
               ZigzagBuffer[shift]=lastlow;
               whatlookfor=1;
              }   
            break;               
         default: return; 
        }
     }


PrevZigBars = ZigBars ;

ZigBars = MathAbs( lastlowpos - lasthighpos ) + 1 ;
xWait   = ZigBars - MathMin( lastlowpos , lasthighpos ) ;
xWait   = MathAbs( xWait ) - 1 ;


n       = 10;
 
j       = 0 ;


lbl[j]  = ShortName+j;
lbl2[j] = ShortName;
DoShowHead(j, colorHead);

j       = j + 1 ;
lbl[j]  = ShortName+j;
lbl2[j] = "ZigZag Top(" + lasthighpos + "): " ;
DoShow(j, Lime,  DoubleToStr(lasthigh, digits) ); 
 

j       = j + 1 ;
lbl[j]  = ShortName+j;
lbl2[j] = "ZigZag Bot(" + lastlowpos + "): " ;
DoShow(j, Red,  DoubleToStr(lastlow, digits)); 

j       = j + 1 ;
lbl[j]  = ShortName+j;
lbl2[j] = "ZigZag Bars:"   ;
DoShow(j, Orange, ZigBars); 

 
j       = j + 1 ;
lbl[j]  = ShortName+j;
lbl2[j] = "WAIT:       "   ;
DoShow(j, Orange, xWait); 

/*

j       = j + 1 ;
lbl[j]  = ShortName+j;
lbl2[j] = "Prev ZigZag:"   ;
DoShow(j, Blue, PrevZigBars); 
*/

WindowRedraw() ; 



   return(0);
  } 


//+------------------------------------------------------------------+
  
void DoShowHead( int u, color dsColor  )
{

ObjectCreate(lbl[u],23,0,Time[0],PRICE_CLOSE);
ObjectSet(lbl[u], OBJPROP_CORNER, myCorner );
ObjectSet(lbl[u],OBJPROP_XDISTANCE, myChartX);
ObjectSet(lbl[u],OBJPROP_YDISTANCE,n+myChartY);
ObjectSetText(lbl[u],lbl2[u],myFontSize,myFont,dsColor);


n = n+20;


  }  

//+------------------------------------------------------------------+
  
void DoShow( int u, color dsColor, string value  )
{

ObjectCreate(lbl[u],23,0,Time[0],PRICE_CLOSE);
ObjectSet(lbl[u], OBJPROP_CORNER, myCorner );
ObjectSet(lbl[u],OBJPROP_XDISTANCE, myChartX);
ObjectSet(lbl[u],OBJPROP_YDISTANCE,n+myChartY);
ObjectSetText(lbl[u],lbl2[u],myFontSize,myFont,dsColor);


string Label01 = lbl[u] + "val" ;
ObjectCreate(Label01,23,0,Time[0],PRICE_CLOSE);
ObjectSet(Label01, OBJPROP_CORNER, myCorner );
ObjectSet(Label01,OBJPROP_XDISTANCE,160+myChartX);
ObjectSet(Label01,OBJPROP_YDISTANCE,n+myChartY);
ObjectSetText(Label01,value,myFontSize,myFont,dsColor);

n = n+20;

}  
  
//+------------------------------------------------------------------+
string TimeFrameToString(int tf)
{
   string tfs;
   switch(tf) {
      case PERIOD_M1:  tfs="M1"  ; break;
      case PERIOD_M5:  tfs="M5"  ; break;
      case PERIOD_M15: tfs="M15" ; break;
      case PERIOD_M30: tfs="M30" ; break;
      case PERIOD_H1:  tfs="H1"  ; break;
      case PERIOD_H4:  tfs="H4"  ; break;
      case PERIOD_D1:  tfs="D1"  ; break;
      case PERIOD_W1:  tfs="W1"  ; break;
      case PERIOD_MN1: tfs="MN";
   }
   return(tfs);
}


//+------------------------------------------------------------------+

/*

Comment( 
        
"lastlowpos = ", lastlowpos, "\n",  
"lasthighpos = ", lasthighpos, "\n",  
"xWait = ", xWait, "\n", 


"") ;

*/